class RoutersploitException(Exception):
    def __init__(self, msg: str = ""):
        super(RoutersploitException, self).__init__(msg)


class OptionValidationError(RoutersploitException):
    pass


class StopThreadPoolExecutor(RoutersploitException):
    pass
